/*==============================================================================*/
/*  Copyright (C) 2009-2016, Heartland Data inc. All Rights Reserved.           */
/*                                                                              */
/*  Title  :   Asynchronous Bus Driver                                          */
/*  EventID:   Via the driver                                                   */
/*  AppVer :   DT10 Ver10.XX～                                                  */
/*  FileID :   31a                                                              */
/*  Version:   1.1                                                              */
/*  Author :   HLDC                                                             */
/*==============================================================================*/

/*==============================================================================*/
/*  Desc:   Header for FROM Control                                             */
/*==============================================================================*/
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <sys/mman.h>
#include <asm/page.h>
#include <sys/types.h> 
#include <unistd.h> 

/*==============================================================================*/
/*  Macro:  DT_UINT                                                             */
/*  Desc:   Please change Test Point argument type for DT10 Project setting.    */
/*==============================================================================*/
#define	DT_UINT unsigned int

/*==============================================================================*/
/*  Desc:   Test Point Parametaer                                               */
/*==============================================================================*/
#define		DT_VARIABLE_BIT			0x02			/* Memory Dump MASK */
#define		DT_EVTTRG_BIT			0x08			/* EventTrigger */

/*==============================================================================*/
/*  Desc:   Please describe the code to Interrupt enable/disable.                */
/*==============================================================================*/
#define		DT_INT_DISABLE()
#define		DT_INT_RESTORE()

/*==============================================================================*/
/*  Macro:  DT_ADD_EVENT_INFO                                                   */
/*  Desc:   Please set 1 when you add Event Value to Test Point.                */
/*==============================================================================*/
#define	DT_ADD_EVENT_INFO 0

/*==============================================================================*/
/*  Func:   addEventInfo                                                        */
/*  Desc:   please describe the code to get target event                        */
/*==============================================================================*/
#if DT_ADD_EVENT_INFO
DT_UINT addEventInfo(void)
{
	static unsigned int event_id = 0;

	/* describe code to get target eventinformation */
	event_id = getpid();
	return event_id;
}
#endif

static int fd = -1;
static void *cookie = NULL;

/*==============================================================================*/
/*  Desc:   Please set beginning address in the virtual memory space.           */
/*==============================================================================*/
#define	FLASH_ADDR	0x60000000


/*==============================================================================*/
/*  Func:   _TP_BusOutDrv                                                       */
/*  Desc:   Test Point Output Function                                          */
/*==============================================================================*/
void _TP_BusOutDrv( unsigned addr, unsigned dat )
{
	if( fd > 0 ){
		cookie = mmap( NULL, PAGE_SIZE, PROT_WRITE | PROT_READ, MAP_SHARED, fd, FLASH_ADDR+ (addr/PAGE_SIZE)*PAGE_SIZE);
		*(volatile unsigned short *)(cookie + (addr&0xfff)) = dat;
		munmap( cookie, PAGE_SIZE );
	}
}

/*==============================================================================*/
/*  Func:   _TP_BusInit                                                         */
/*  Desc:   mem bus init                                                        */
/*==============================================================================*/
void _TP_BusInit(void)
{
	if( fd < 0 )
	{
		fd = open("/dev/mem", O_RDWR);
		if( fd < 0 ){
			printf( "Can't open /dev/mem\n" );
		}
	}
}

/*==============================================================================*/
/*  Func:   _TP_BusOut                                                          */
/*  Desc:   Called by Test Point                                                */
/*==============================================================================*/
void _TP_BusOut( DT_UINT addr, DT_UINT dat )
{
#if DT_ADD_EVENT_INFO
	unsigned int event_id =0;
#endif

/*
The Linux has the file access system for memory space: “/dev/mem”; so you should 
mmap to “/dev/mem”.
	
Please modify the 2nd argument PAGE_SIZE and the 6th argument: initial address of 
FlashROM of mmap function matching to your target environment.
*/
	_TP_BusInit();

#if DT_ADD_EVENT_INFO
	event_id = addEventInfo();
	_TP_BusOutDrv( addr | DT_EVTTRG_BIT, dat );
	_TP_BusOutDrv( DT_EVTTRG_BIT, event_id );
	_TP_BusOutDrv( DT_EVTTRG_BIT, event_id >> 16 );
#else
	_TP_BusOutDrv( addr, dat );
#endif

/*
	cookie = mmap( NULL, PAGE_SIZE, PROT_WRITE | PROT_READ, MAP_SHARED, fd,
					FLASH_ADDR + ((addr / PAGEG_SIZE) * PAGE_SIZE) );
	if (cookie == MAP_FAILED) {
		printf("mmap error\n");
		return;
	}
#if DT_ADD_EVENT_INFO
	p = cookie + ((addr | DT_EVTTRG_BIT) % PAGE_SIZE);
	*(unsigned short *)p = dat;
	event_id = addEventInfo();
	*(unsigned short *)p = (event_id & 0x0000ffff);
	*(unsigned short *)p = ((event_id >> 16) & 0x0000ffff));
#else
	p = (unsigned int)cookie + (addr % PAGE_SIZE);
	*(unsigned short *)p = dat;
#endif
	munmap(cookie, PAGE_SIZE );
*/
}

/*==============================================================================*/
/*  Func:   _TP_MemoryOutput                                                    */
/*  Desc:   Called by Variable TestPoint                                        */
/*==============================================================================*/
void _TP_MemoryOutput( DT_UINT addr, DT_UINT dat, void *value, DT_UINT size)
{
	DT_UINT i;
	unsigned char *p;
#if DT_ADD_EVENT_INFO
	unsigned int event_id =0;
#endif

	_TP_BusInit();

	DT_INT_DISABLE();								/* Disable Interrupt */
#if DT_ADD_EVENT_INFO
	_TP_BusOutDrv( addr | DT_VARIABLE_BIT | DT_EVTTRG_BIT, dat);		/* Memory Dump Start */
#else
	_TP_BusOutDrv( addr | DT_VARIABLE_BIT, dat);	/* Memory Dump Start */
#endif
	
#if DT_ADD_EVENT_INFO
	event_id = addEventInfo();
	_TP_BusOutDrv( DT_EVTTRG_BIT, event_id );
	_TP_BusOutDrv( DT_EVTTRG_BIT, event_id >> 16 );
#endif
	if (size >= 256) size = 256;
	if(value){
		p = (unsigned char *)value;
		for (i = 0; i < size; i += 2) {
			dat = p[i];
			if ((i + 1) < size) {
				dat |= p[i+1] << 8;
			}
			addr = DT_VARIABLE_BIT;			/* Variable Bit */
			if (i == 0) {
				addr |= size << 8;			/* size */
			}
			_TP_BusOutDrv( addr, dat );
		}
	}
	DT_INT_RESTORE();								/* Enable Interrupt */ 
}

/*==============================================================================*/
